QuestAnnounce = LibStub("AceAddon-3.0"):NewAddon("QuestAnnounce", "AceConsole-3.0", "AceEvent-3.0")
local QuestAnnounce = _G.QuestAnnounce
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local AceLocale = LibStub("AceLocale-3.0")
local L = AceLocale:GetLocale("QuestAnnounce")

--[[ The defaults user without a profile will get. --]]
local defaults = {
	char={
		Enable = true,
		Debug = false,
		Every = 1
	}
}

local function getCharOption(info)
	QuestAnnounce:SendDebugMsg("getCharValue: "..info.arg.." :: "..tostring(QuestAnnounce.db.char[info.arg]))
	return QuestAnnounce.db.char[info.arg]
end

local function setCharOption(info, value)
	QuestAnnounce:SendDebugMsg("setCharValue: "..info.arg.." :: "..tostring(value))
	
	QuestAnnounce.db.char[info.arg] = value
	if(info.uiType == "cmd") then
		QuestAnnounce:Print(info.arg.." - "..tostring(value))
	end
end

--[[ The command line options. This options table is also used in the GUI config. --]] 
local options = {
    type = 'group',
	get = getCharOption,
	set = setCharOption,
    args = {
		settings = {
			type = 'group',
			name = L["Settings"],
			cmdInline = true,
			order = -1,
			args = {
				[L["enable"]] = {
					type = 'toggle',
					order = 1,
					name = L["Enable"],
					desc = L["Enables/Disables Quest Announce"],
					arg = "Enable",
					get = getCharOption,
					set = setCharOption,
		        },
				[L["every"]] = {
					type = 'range',
					order = 2,
					name = L["Announce Every"],
					desc = L["Announce progression every x number of steps"],
					min = 1,
					max = 10,
					step = 1,
					arg = "Every",
					get = getCharOption,
					set = setCharOption,
					validate = function(info, value) 
						if(info.uiType == "cmd") and ((value > 10) or (value < 1)) then 
							return L["Announce Every has to be between 1 and 10"] 
						else
							return true
						end
					end
		        },
				[L["debug"]] = {
					type = 'toggle',
					order = 10,
					name = L["Debug"],
					desc = L["Enables/Disables Quest Announce Debug Mode"],
					hidden = true,
					arg = "Debug",
					get = getCharOption,
					set = setCharOption
		        },
				[L["status"]] = {
					type = 'execute',
					order = 3,
					name = L["Status"],
					desc = L["Display the current status of Quest Announce"],
					func = function()
						QuestAnnounce:Print(L["Status"].."\n    "..
							L["Enable"].." - "..(QuestAnnounce.db.char.Enable and L["Enabled"] or L["Disabled"]).."\n    "..
							L["Announce Every"].." - "..tostring(QuestAnnounce.db.char.Every))
					end
		        },
				[L["reset"]] = {
					type = 'execute',
					order = 4,
					name = L["Reset"],
					desc = L["Resets your Quest Announce profile to the default settings"],
					func = function() 
						QuestAnnounce.db:ResetDB("default")
						QuestAnnounce:Print(L["Settings have been reset to the defaults"])
					end
		        }
			}
		}
    }
}

function QuestAnnounce:OnInitialize()
	local AceDB = LibStub:GetLibrary("AceDB-3.0")
	QuestAnnounce.db = AceDB:New("QuestAnnounceDB", defaults)
	--[[ LIVE ]]--
	
	--[[ We're looking at the UI_INFO_EVENT for quest messages ]]--	
	--QuestAnnounce:RegisterEvent("UI_INFO_MESSAGE", function(event, ...) QuestAnnounce:OnEvent(event, ...) end)
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("QuestAnnounce", options, {"qa", "questannounce"})

	--[[ Blizzard Addon Interface ]]--
	QuestAnnounce.optionsFrames = {}
	QuestAnnounce.optionsFrames.general = AceConfigDialog:AddToBlizOptions("QuestAnnounce", "Quest Announce", nil, "settings")
end

function QuestAnnounce:OnEnable()
	--[[ We're looking at the UI_INFO_EVENT for quest messages ]]--	
	QuestAnnounce:RegisterEvent("UI_INFO_MESSAGE", function(event, ...) QuestAnnounce:OnEvent(event, ...) end)
	
	QuestAnnounce:SendDebugMsg(tostring(QuestAnnounce.db.char.Enable))
end

function QuestAnnounce:OnEvent(event, ...)
	local msg = ...
	if (msg ~= nil) then
		if (QuestAnnounce.db.char.Enable) then
			local questText = gsub(msg,"(.*):%s*([-%d]+)%s*/%s*([-%d]+)%s*$", "%1", 1)
			
			if (questText ~= msg) then
				local ii, jj, strItemName, iNumItems, iNumNeeded = string.find(msg, "(.*):%s*([-%d]+)%s*/%s*([-%d]+)%s*$")
				local stillNeeded = iNumNeeded - iNumItems
                
				local every = math.fmod(iNumItems, QuestAnnounce.db.char.Every)
				QuestAnnounce:SendDebugMsg("Item Name: "..strItemName.." :: Num Items: "..iNumItems.." :: Num Needed: "..iNumNeeded.." :: Every fMod: "..every)
				
				if(every == 0 and stillNeeded > 0 ) then
					QuestAnnounce:SendPartyMsg(L["Progress: "]..msg)
				elseif(stillNeeded == 0) then
					QuestAnnounce:SendPartyMsg(L["Completed: "]..msg)
				end
			end
		end
	end
end

function QuestAnnounce:SendDebugMsg(msg)
	if(msg ~= nil and QuestAnnounce.db.char.Debug) then
		QuestAnnounce:Print("DEBUG :: "..msg)
	end
end

function QuestAnnounce:SendPartyMsg(msg)	
	if (msg ~= nil and QuestAnnounce.db.char.Enable and GetNumPartyMembers() > 0) then
		SendChatMessage(msg, "PARTY")
	end
	QuestAnnounce:SendDebugMsg("QuestAnnounce:SendPartyMsg - "..msg)
end


